=== modified file 'drizzled/db.cc'
--- drizzled/db.cc	2009-06-11 21:19:36 +0000
+++ drizzled/db.cc	2009-07-09 01:35:10 +0000
@@ -34,6 +34,7 @@
 #include <drizzled/lock.h>
 #include <drizzled/errmsg_print.h>
 #include <drizzled/transaction_services.h>
+#include <drizzled/message/schema.pb.h>
 
 extern drizzled::TransactionServices transaction_services;
 
@@ -72,16 +73,31 @@
 
 const CHARSET_INFO *get_default_db_collation(const char *db_name)
 {
-  HA_CREATE_INFO db_info;
-
-  /*
-    db_info.default_table_charset contains valid character set
-    (collation_server).
-  */
-
-  load_db_opt_by_name(db_name, &db_info);
-
-  return db_info.default_table_charset;
+  drizzled::message::Schema db;
+
+  get_database_metadata(db_name, &db);
+
+  /* If for some reason the db.opt file lacks a collation,
+     we just return the default */
+
+  if (db.has_collation())
+  {
+    const string buffer= db.collation();
+    const CHARSET_INFO* cs= get_charset_by_name(buffer.c_str());
+
+    if (!cs)
+    {
+      errmsg_printf(ERRMSG_LVL_ERROR,
+                    _("Error while loading database options: '%s':"),db_name);
+      errmsg_printf(ERRMSG_LVL_ERROR, ER(ER_UNKNOWN_COLLATION), buffer.c_str());
+
+      return default_charset_info;
+    }
+
+    return cs;
+  }
+
+  return default_charset_info;
 }
 
 /* path is path to database, not schema file */
@@ -130,44 +146,7 @@
   return 0;
 }
 
-int load_db_opt(const char *path, HA_CREATE_INFO *create)
-{
-  drizzled::message::Schema db;
-
-  memset(create, 0, sizeof(*create));
-  create->default_table_charset= default_charset_info;
-
-  int fd= open(path, O_RDONLY);
-
-  if (fd == -1)
-    return errno;
-
-  if (!db.ParseFromFileDescriptor(fd))
-  {
-    close(fd);
-    return -1;
-  }
-  close(fd);
-
-  /* If for some reason the db.opt file lacks a collation, we just return the default */
-  if (db.has_collation())
-  {
-    string buffer;
-    buffer= db.collation();
-    if (!(create->default_table_charset= get_charset_by_name(buffer.c_str())))
-    {
-      errmsg_printf(ERRMSG_LVL_ERROR,
-                    _("Error while loading database options: '%s':"),path);
-      errmsg_printf(ERRMSG_LVL_ERROR, ER(ER_UNKNOWN_COLLATION), buffer.c_str());
-      create->default_table_charset= default_charset_info;
-      return -1;
-    }
-  }
-
-  return 0;
-}
-
-int load_db_opt_by_name(const char *db_name, HA_CREATE_INFO *db_create_info)
+int get_database_metadata(const char *dbname, drizzled::message::Schema *db)
 {
   char db_opt_path[FN_REFLEN];
   size_t length;
@@ -177,13 +156,24 @@
     to avoid table name to file name encoding.
   */
   length= build_table_filename(db_opt_path, sizeof(db_opt_path),
-                              db_name, "", false);
+                              dbname, "", false);
   strcpy(db_opt_path + length, MY_DB_OPT_FILE);
 
-  return load_db_opt(db_opt_path, db_create_info);
+  int fd= open(db_opt_path, O_RDONLY);
+
+  if (fd == -1)
+    return errno;
+
+  if (!db->ParseFromFileDescriptor(fd))
+  {
+    close(fd);
+    return -1;
+  }
+  close(fd);
+
+  return 0;
 }
 
-
 /*
   Create a database
 

=== modified file 'drizzled/db.h'
--- drizzled/db.h	2009-06-05 11:49:01 +0000
+++ drizzled/db.h	2009-07-09 01:35:10 +0000
@@ -21,6 +21,8 @@
 #ifndef DRIZZLE_SERVER_DB_H
 #define DRIZZLE_SERVER_DB_H
 
+namespace drizzled { namespace message { class Schema; } }
+
 bool mysql_create_db(Session *session, const char *db, HA_CREATE_INFO *create_info);
 bool mysql_alter_db(Session *session, const char *db, HA_CREATE_INFO *create);
 bool mysql_rm_db(Session *session,char *db, bool if_exists);
@@ -28,7 +30,8 @@
                      bool force_switch);
 
 bool check_db_dir_existence(const char *db_name);
-int load_db_opt_by_name(const char *db_name, HA_CREATE_INFO *db_create_info);
+int get_database_metadata(const char *dbname, drizzled::message::Schema *db);
+
 const CHARSET_INFO *get_default_db_collation(const char *db_name);
 
 extern int creating_database; // How many database locks are made

=== modified file 'drizzled/show.cc'
--- drizzled/show.cc	2009-07-03 19:45:58 +0000
+++ drizzled/show.cc	2009-07-09 01:35:10 +0000
@@ -42,6 +42,7 @@
 #include <drizzled/item/empty_string.h>
 #include "drizzled/plugin_registry.h"
 #include <drizzled/info_schema.h>
+#include <drizzled/message/schema.pb.h>
 
 #include <string>
 #include <iostream>
@@ -282,14 +283,71 @@
   return false;
 }
 
-bool mysqld_show_create_db(Session *session, char *dbname,
-                           HA_CREATE_INFO *create_info)
+/**
+  Get a CREATE statement for a given database.
+
+  The database is identified by its name, passed as @c dbname parameter.
+  The name should be encoded using the system character set (UTF8 currently).
+
+  Resulting statement is stored in the string pointed by @c buffer. The string
+  is emptied first and its character set is set to the system character set.
+
+  If HA_LEX_CREATE_IF_NOT_EXISTS flag is set in @c create_info->options, then
+  the resulting CREATE statement contains "IF NOT EXISTS" clause. Other flags
+  in @c create_options are ignored.
+
+  @param  session           The current thread instance.
+  @param  dbname        The name of the database.
+  @param  buffer        A String instance where the statement is stored.
+  @param  create_info   If not NULL, the options member influences the resulting
+                        CRATE statement.
+
+  @returns true if errors are detected, false otherwise.
+*/
+
+static bool store_db_create_info(const char *dbname, String *buffer, bool if_not_exists)
+{
+  drizzled::message::Schema schema;
+
+  if (!my_strcasecmp(system_charset_info, dbname,
+                     INFORMATION_SCHEMA_NAME.c_str()))
+  {
+    dbname= INFORMATION_SCHEMA_NAME.c_str();
+  }
+  else
+  {
+    int r= get_database_metadata(dbname, &schema);
+    if(r < 0)
+      return true;
+  }
+
+  buffer->length(0);
+  buffer->free();
+  buffer->set_charset(system_charset_info);
+  buffer->append(STRING_WITH_LEN("CREATE DATABASE "));
+
+  if (if_not_exists)
+    buffer->append(STRING_WITH_LEN("IF NOT EXISTS "));
+
+  buffer->append_identifier(dbname, strlen(dbname));
+
+  if (schema.has_collation() && strcmp(schema.collation().c_str(),
+                                       default_charset_info->name))
+  {
+    buffer->append(" COLLATE = ");
+    buffer->append(schema.collation().c_str());
+  }
+
+  return false;
+}
+
+bool mysqld_show_create_db(Session *session, char *dbname, bool if_not_exists)
 {
   char buff[2048];
   String buffer(buff, sizeof(buff), system_charset_info);
   Protocol *protocol=session->protocol;
 
-  if (store_db_create_info(dbname, &buffer, create_info))
+  if (store_db_create_info(dbname, &buffer, if_not_exists))
   {
     /*
       This assumes that the only reason for which store_db_create_info()
@@ -768,60 +826,6 @@
   return(0);
 }
 
-/**
-  Get a CREATE statement for a given database.
-
-  The database is identified by its name, passed as @c dbname parameter.
-  The name should be encoded using the system character set (UTF8 currently).
-
-  Resulting statement is stored in the string pointed by @c buffer. The string
-  is emptied first and its character set is set to the system character set.
-
-  If HA_LEX_CREATE_IF_NOT_EXISTS flag is set in @c create_info->options, then
-  the resulting CREATE statement contains "IF NOT EXISTS" clause. Other flags
-  in @c create_options are ignored.
-
-  @param  session           The current thread instance.
-  @param  dbname        The name of the database.
-  @param  buffer        A String instance where the statement is stored.
-  @param  create_info   If not NULL, the options member influences the resulting
-                        CRATE statement.
-
-  @returns true if errors are detected, false otherwise.
-*/
-
-bool store_db_create_info(const char *dbname, String *buffer, HA_CREATE_INFO *create_info)
-{
-  HA_CREATE_INFO create;
-  uint32_t create_options = create_info ? create_info->options : 0;
-
-  if (!my_strcasecmp(system_charset_info, dbname,
-                     INFORMATION_SCHEMA_NAME.c_str()))
-  {
-    dbname= INFORMATION_SCHEMA_NAME.c_str();
-    create.default_table_charset= system_charset_info;
-  }
-  else
-  {
-    if (check_db_dir_existence(dbname))
-      return true;
-
-    load_db_opt_by_name(dbname, &create);
-  }
-
-  buffer->length(0);
-  buffer->free();
-  buffer->set_charset(system_charset_info);
-  buffer->append(STRING_WITH_LEN("CREATE DATABASE "));
-
-  if (create_options & HA_LEX_CREATE_IF_NOT_EXISTS)
-    buffer->append(STRING_WITH_LEN("IF NOT EXISTS "));
-
-  buffer->append_identifier(dbname, strlen(dbname));
-
-  return false;
-}
-
 static void store_key_options(String *packet, Table *table, KEY *key_info)
 {
   char *end, buff[32];

=== modified file 'drizzled/show.h'
--- drizzled/show.h	2009-07-02 06:54:37 +0000
+++ drizzled/show.h	2009-07-09 01:35:10 +0000
@@ -67,7 +67,7 @@
                  LOOKUP_FIELD_VALUES *lookup_field_vals, bool *with_i_schema);
 
 int store_create_info(TableList *table_list, String *packet, HA_CREATE_INFO  *create_info_arg);
-bool store_db_create_info(const char *dbname, String *buffer, HA_CREATE_INFO *create_info);
+
 bool schema_table_store_record(Session *session, Table *table);
 
 int get_quote_char_for_identifier();
@@ -85,7 +85,7 @@
 void mysqld_list_fields(Session *session,TableList *table, const char *wild);
 int mysqld_dump_create_info(Session *session, TableList *table_list, int fd);
 bool drizzled_show_create(Session *session, TableList *table_list);
-bool mysqld_show_create_db(Session *session, char *dbname, HA_CREATE_INFO *create);
+bool mysqld_show_create_db(Session *session, char *dbname, bool if_not_exists);
 
 int mysqld_show_status(Session *session);
 int mysqld_show_variables(Session *session,const char *wild);

=== modified file 'drizzled/sql_parse.cc'
--- drizzled/sql_parse.cc	2009-06-30 18:31:32 +0000
+++ drizzled/sql_parse.cc	2009-07-09 01:35:10 +0000
@@ -1180,7 +1180,9 @@
       my_error(ER_WRONG_DB_NAME, MYF(0), lex->name.str);
       break;
     }
-    res= mysqld_show_create_db(session, lex->name.str, &lex->create_info);
+    res= mysqld_show_create_db(session, lex->name.str,
+                               lex->create_info.options &
+                                 HA_LEX_CREATE_IF_NOT_EXISTS);
     break;
   }
   case SQLCOM_FLUSH:

=== modified file 'plugin/info_schema/info_schema_methods.cc'
--- plugin/info_schema/info_schema_methods.cc	2009-07-02 06:54:37 +0000
+++ plugin/info_schema/info_schema_methods.cc	2009-07-09 01:35:10 +0000
@@ -635,11 +635,9 @@
     }
     else
     {
-      HA_CREATE_INFO create;
-      load_db_opt_by_name((*db_name)->str, &create);
+      const CHARSET_INFO *cs= get_default_db_collation((*db_name)->str);
 
-      if (store_schema_schemata(session, table, *db_name,
-                               create.default_table_charset))
+      if (store_schema_schemata(session, table, *db_name, cs))
         return(1);
     }
 

=== modified file 'tests/r/alter_database.result'
--- tests/r/alter_database.result	2009-06-05 12:04:43 +0000
+++ tests/r/alter_database.result	2009-07-09 01:35:10 +0000
@@ -5,9 +5,9 @@
 alter database foo collate = utf8_bin;
 show create database foo;
 Database	Create Database
-foo	CREATE DATABASE `foo`
+foo	CREATE DATABASE `foo` COLLATE = utf8_bin
 alter database foo collate = utf8_esperanto_ci;
 show create database foo;
 Database	Create Database
-foo	CREATE DATABASE `foo`
+foo	CREATE DATABASE `foo` COLLATE = utf8_esperanto_ci
 drop database foo;

=== modified file 'tests/r/default_collation.result'
--- tests/r/default_collation.result	2009-05-02 23:57:11 +0000
+++ tests/r/default_collation.result	2009-07-09 01:35:10 +0000
@@ -1,4 +1,7 @@
 create database mysqltest collate = utf8_bin;
+show create database mysqltest;
+Database	Create Database
+mysqltest	CREATE DATABASE `mysqltest` COLLATE = utf8_bin
 use mysqltest;
 create table t1 (a varchar(10));
 create table t2 (a varchar(10)) collate = utf8_general_ci;

=== modified file 'tests/r/drizzledump.result'
--- tests/r/drizzledump.result	2009-06-05 23:10:06 +0000
+++ tests/r/drizzledump.result	2009-07-09 01:35:10 +0000
@@ -126,7 +126,7 @@
 SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;
 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
 
-CREATE DATABASE IF NOT EXISTS `mysqldump_test_db`;
+CREATE DATABASE IF NOT EXISTS `mysqldump_test_db` COLLATE = utf8_esperanto_ci;
 
 USE `mysqldump_test_db`;
 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;

=== modified file 'tests/t/default_collation.test'
--- tests/t/default_collation.test	2009-05-02 23:57:11 +0000
+++ tests/t/default_collation.test	2009-07-09 01:35:10 +0000
@@ -8,6 +8,7 @@
 # assuming system default collation is utf8_general_ci
 
 create database mysqltest collate = utf8_bin;
+show create database mysqltest;
 
 use mysqltest;
 

